// !Together v1.22

// Keeps paragraphs together.
// Written by Roger Noble, November 2000

// Version History:
// v1.00 (30th November 2000)  :  Initial version
// v1.01 (6th December 2000)   :  Handled the case when already at the start of a new page,
//                                and so we do not want to insert yet another one.
//                                Fixed error in bookmark low and high water setting.
//                                Deferred calling set_styles when a new view is opened:
//                                the document styles do not appear to be set up at this
//                                stage.
// v1.02 (6th April 2001)      :  Display hour glass when busy.
// v1.10 (11th June 2001)      :  Added keep paragraph together, widow and orphan control, and top of page option.
//                                Keep with next now checks the start of the paragraph rather than the end when as-you-type
//                                selected. (This was previously inconsistent with the Scan action.)
// v1.11 (18th June 2001)      :  Fixed so that it checks all styles that might match. It was only returning the first.
// v1.12 (9th July 2001)       :  Corrected fault in orphan handling when doing a scan. The code was incorrectly regarding
//                                a single line paragraph on its own at the end of page as a orphan.
// v1.20 (20th July 2001)      :  Added support for !!!!!!!bm, so that we tidy ourselves up if any of
//                                our bookmarks are deleted by the user.
// v1.21 (3rd August 2001)     :  Corrected same fault as in v1.12, but for key press handling this time. Funny, I
//                                thought that I had checked this ...
// V1.22 (4th August 2001)     :  Removed delete word keypress handling etc. 7!bm does this for us by selecting the
//                                approriate zone and calling our zone delete handler.


//=============================================================================================

// Global Variables.

// Handles and techie things.

int Together_stylemenu;     // Style menu-button menu handle
int Together_win;           // The options window handle

// Styles.

int Together_Style1 = -1;   // Style handles
int Together_Style2 = -1;
int Together_Style3 = -1;
int Together_Style4 = -1;
int Together_Style5 = -1;
int Together_Style6 = -1;
string Together_Style1s;    // Style names
string Together_Style2s;
string Together_Style3s;
string Together_Style4s;
string Together_Style5s;
string Together_Style6s;
int Together_temp1 = -1;    // Used in Together dialogue to hold style handles
int Together_temp2 = -1;
int Together_temp3 = -1;
int Together_temp4 = -1;
int Together_temp5 = -1;
int Together_temp6 = -1;
int Together_Mode1 = 0;     // Mode, 0 = inactive, 1 = with next, 2 = with previous, 4 = paragraph,
                            // 8 = widow control, 16 = orphan control, 32 = top of page (can be ORed)
int Together_Mode2 = 0;
int Together_Mode3 = 0;
int Together_Mode4 = 0;
int Together_Mode5 = 0;
int Together_Mode6 = 0;
int Together_Mode1_temp = 0;  // Used in Together dialogue to hold temporary values
int Together_Mode2_temp = 0;
int Together_Mode3_temp = 0;
int Together_Mode4_temp = 0;
int Together_Mode5_temp = 0;
int Together_Mode6_temp = 0;

// Options.

int Together_as_you_type = 1;       // Keep together as-you-type
int Together_as_you_type_temp = 1;
int Together_all_chapters = 0;
int Together_all_chapters_temp = 0;

// New page marker bookmarks.

int Together_high_water = 1;	// Highest new page bookmark number.
int Together_low_water = 1;	// Lowest new page bookmark number.

// Bookmark number used when stepping through new pages we did.

int Together_current_bm = 0;
int Together_current_bm_index; // Current bookmark index as last found

// Event handler states.

int Together_eh_installed = 0; // Whether or not the keypress event handler is installed
int Together_styles_set = 0;   // Whether or not the styles have been set up

// Window icons.

int Together_current_icon = 0; // Which style am I currently editing
                               // Bad style to have it global, but this is the simplest
                               // way of getting the job done
// Document-related stuff.

int Together_current_file = 0; 			       // Handle to current file
int Together_current_view = 0;			       // Handle to current view

// Constants.

int Together_tag = ('t'<<24) | ('o'<<16) | ('g'<<8);   // Base tag value

//=============================================================================================

// Find a style with the name "stylename", return the handle for it. Returns -1
// if the style is not found.

int Together_findstyle(string stylename)
{
  int temp;
  string tempstring;

  for (temp = 0; getuserstyle(temp, tempstring, 0) != 0; temp++)
  {
    if (tempstring == stylename)
      return(temp);
  }

  return -1;
}

//---------------------------------------------------------------------------------------------

// Sets the style handles from the style names.

void Together_set_styles()
{
  Together_Style1 = Together_findstyle (Together_Style1s);
  Together_Style2 = Together_findstyle (Together_Style2s);
  Together_Style3 = Together_findstyle (Together_Style3s);
  Together_Style4 = Together_findstyle (Together_Style4s);
  Together_Style5 = Together_findstyle (Together_Style5s);
  Together_Style6 = Together_findstyle (Together_Style6s);
  Together_temp1 = Together_Style1;
  Together_temp2 = Together_Style2;
  Together_temp3 = Together_Style3;
  Together_temp4 = Together_Style4;
  Together_temp5 = Together_Style5;
  Together_temp6 = Together_Style6;
  Together_Mode1_temp = Together_Mode1;
  Together_Mode2_temp = Together_Mode2;
  Together_Mode3_temp = Together_Mode3;
  Together_Mode4_temp = Together_Mode4;
  Together_Mode5_temp = Together_Mode5;
  Together_Mode6_temp = Together_Mode6;

  Together_styles_set = 1;
}

//---------------------------------------------------------------------------------------------

// Test if the style is one of the styles to be kept together starting after the supplied
// style number. Returns the style number, or zero if not.

int Together_isstyle(int style)
{
  if (!Together_styles_set) Together_set_styles(); // Set the style numbers if not done

  caretcontext();
  iscurrentstyle(0);			      // Set up styles

  switch (++style)
  {
    case 1:
      if (iscurrentstyle(Together_Style1) != 0)   // Is it style 1 at the caret?
        return 1;                                 // If so return 1
    case 2:
      if (iscurrentstyle(Together_Style2) != 0)   // and so on
        return 2;
    case 3:
      if (iscurrentstyle(Together_Style3) != 0)
        return 3;
    case 4:
      if (iscurrentstyle(Together_Style4) != 0)
        return 4;
    case 5:
      if (iscurrentstyle(Together_Style5) != 0)
        return 5;
    case 6:
      if (iscurrentstyle(Together_Style6) != 0)
        return 6;
  }

  return 0;                       // It wasn't any of the known styles
}

//---------------------------------------------------------------------------------------------

// Save the current options with the document.

void Together_save_options()
{
  tagwrite (Together_current_file, Together_tag+0, Together_as_you_type, 0);
  tagwrite (Together_current_file, Together_tag+1, Together_all_chapters, 0);
  tagwrite (Together_current_file, Together_tag+2, Together_high_water, 0);
  tagwrite (Together_current_file, Together_tag+3, Together_low_water, 0);
  tagwrites (Together_current_file, Together_tag+4, Together_Style1s);
  tagwrite (Together_current_file, Together_tag+5, Together_Mode1, 0);
  tagwrites (Together_current_file, Together_tag+6, Together_Style2s);
  tagwrite (Together_current_file, Together_tag+7, Together_Mode2, 0);
  tagwrites (Together_current_file, Together_tag+8, Together_Style3s);
  tagwrite (Together_current_file, Together_tag+9, Together_Mode3, 0);
  tagwrites (Together_current_file, Together_tag+10, Together_Style4s);
  tagwrite (Together_current_file, Together_tag+11, Together_Mode4, 0);
  tagwrites (Together_current_file, Together_tag+12, Together_Style5s);
  tagwrite (Together_current_file, Together_tag+13, Together_Mode5, 0);
  tagwrites (Together_current_file, Together_tag+14, Together_Style6s);
  tagwrite (Together_current_file, Together_tag+15, Together_Mode6, 0);
}

//---------------------------------------------------------------------------------------------

// Checks to see if there are any new pages inserted by us. Sets the window
// buttons accordingly. Returns 1 if there are any new pages, else 0.

int Together_check_any_pages()
{
  if (Together_low_water < Together_high_water)
  {
    set_icon_state(Together_win, 62, 0x00000000, 0x00400000);
    if (Together_current_bm > 0)
      set_icon_state(Together_win, 71, 0x00000000, 0x00400000);
    else
      set_icon_state(Together_win, 71, 0x00400000, 0x00400000);
    set_icon_state(Together_win, 58, 0x00000000, 0x00400000);
    return 1;
  }
  else
  {
    set_icon_state(Together_win, 62, 0x00400000, 0x00400000);
    set_icon_state(Together_win, 71, 0x00400000, 0x00400000);
    set_icon_state(Together_win, 58, 0x00400000, 0x00400000);
    return 0;
  }
}
//---------------------------------------------------------------------------------------------

// Checks if any Together styles have been set and that we are likely to have
// some Together bookmarks set.
// Returns 1 if any styles have been set, 0 otherwise.

int Together_check_ready()
{
  int state;
  int i;

  if ((Together_temp1 >= 0 && Together_Mode1_temp != 0) || (Together_temp2 >= 0 && Together_Mode2_temp != 0) ||
      (Together_temp3 >= 0 && Together_Mode3_temp != 0) || (Together_temp4 >= 0 && Together_Mode4_temp != 0) ||
      (Together_temp5 >= 0 && Together_Mode5_temp != 0) || (Together_temp6 >= 0 && Together_Mode6_temp != 0))
  {
    // We can Scan. Enable the Scan action.

    set_icon_state (Together_win, 0, 0x00000000, 0x00400000);
    state = 1;
  }
  else
  {
    // No Together styles or modes are set.
    // Disable the Scan action.

    set_icon_state (Together_win, 0, 0x00400000, 0x00400000);
    state = 0;
  }

  // Set the mode options.

  if (Together_temp1 <= -1)
  {
    for (i = 17; i < 23; ++i)
    {
      select_icon(Together_win, i, 0);
      set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    Together_Mode1_temp = 0;
  }
  else
  {
    if (Together_Mode1_temp & 32)
    {
      for (i = 17; i < 22; ++i)
        set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    else
    {
      for (i = 17; i < 23; ++i)
        set_icon_state(Together_win, i, 0x00000000, 0x00400000);
    }
  }

  if (Together_temp2 <= -1)
  {
    for (i = 23; i < 29; ++i)
    {
      select_icon(Together_win, i, 0);
      set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    Together_Mode2_temp = 0;
  }
  else
  {
    if (Together_Mode2_temp & 32)
    {
      for (i = 23; i < 28; ++i)
        set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    else
    {
      for (i = 23; i < 29; ++i)
        set_icon_state(Together_win, i, 0x00000000, 0x00400000);
    }
  }

  if (Together_temp3 <= -1)
  {
    for (i = 29; i < 35; ++i)
    {
      select_icon(Together_win, i, 0);
      set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    Together_Mode3_temp = 0;
  }
  else
  {
    if (Together_Mode3_temp & 32)
    {
      for (i = 29; i < 34; ++i)
        set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    else
    {
      for (i = 29; i < 35; ++i)
        set_icon_state(Together_win, i, 0x00000000, 0x00400000);
    }
  }

  if (Together_temp4 <= -1)
  {
    for (i = 35; i < 41; ++i)
    {
      select_icon(Together_win, i, 0);
      set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    Together_Mode4_temp = 0;
  }
  else
  {
    if (Together_Mode4_temp & 32)
    {
      for (i = 35; i < 40; ++i)
        set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    else
    {
      for (i = 35; i < 41; ++i)
        set_icon_state(Together_win, i, 0x00000000, 0x00400000);
    }
  }

  if (Together_temp5 <= -1)
  {
    for (i = 41; i < 47; ++i)
    {
      select_icon(Together_win, i, 0);
      set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    Together_Mode5_temp = 0;
  }
  else
  {
    if (Together_Mode5_temp & 32)
    {
      for (i = 41; i < 46; ++i)
        set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    else
    {
      for (i = 41; i < 47; ++i)
        set_icon_state(Together_win, i, 0x00000000, 0x00400000);
    }
  }

  if (Together_temp6 <= -1)
  {
    for (i = 47; i < 53; ++i)
    {
      select_icon(Together_win, i, 0);
      set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    Together_Mode6_temp = 0;
  }
  else
  {
    if (Together_Mode6_temp & 32)
    {
      for (i = 47; i < 52; ++i)
        set_icon_state(Together_win, i, 0x00400000, 0x00400000);
    }
    else
    {
      for (i = 47; i < 53; ++i)
        set_icon_state(Together_win, i, 0x00000000, 0x00400000);
    }
  }

  Together_check_any_pages();

  return state;
}

//---------------------------------------------------------------------------------------------

// Inserts a new page at the location of the supplied bookmark, together
// with a Together new page bookmark before the new page to indicate that
// it is a new page that we inserted.

void Together_insert_page(int bm)
{
  int tempbm;
  int page;

  setcarettobm(bm);  // Move the caret to the supplied bookmark

  // Insert the new page, but only if there isn't one there already.

  if (caretchar() != 3)
  {
    // Check if we are already at the top of a page. If so, then we
    // do not want to insert a new page. We check by going to the end
    // of the previous paragraph and checking it that is on the same
    // page as our current one. If it is, then we can insert the new
    // page; if it is not then we should not.

    tempbm = bmcreate("Together_temp");
    setbmtobm(tempbm, bm);
    if (bmmove(tempbm, 0, 7) != 0) // If not at beginning of text
    {
      bmmove(tempbm, 1, 3);        // Now at end of previous paragraph
      page = bmpage(tempbm);
      bmdelete(tempbm);
      if (page == bmpage(bm))
      {
        // We really can insert the new page.

        type("{NewPage}");

        // Create a new specially-named bookmark to indicate the location
        // of the new page.

        tempbm = bmcreate("TogE" + itos(Together_high_water));
        Together_high_water++;
        type("{CLeft}");        // Move back over the new page
        setbmtocaret(tempbm);   // Set the bookmark

        Together_save_options(); // Save the options (and the high and low water marks)
        Together_check_any_pages();
      }
    }
    else
      bmdelete(tempbm);
  }
}

//---------------------------------------------------------------------------------------------

// Called on a key press. If it is a return (for a new paragraph) then we check the style
// of the current paragraph. If it is one of those that we manage, then we make sure
// that it is kept together as required.

int Together_keypress(int user, int view, int key)
{
  int bm, bm1;
  int style = 0;
  int mode = 0;
  int page;

  if (zoneorder() != 0) return key;

  if (key != 13) return key;

  style = Together_isstyle(style);
  while (style != 0)
  {
    switch (style)
    {
      case 1:
        mode |= Together_Mode1;
        break;
      case 2:
        mode |= Together_Mode2;
        break;
      case 3:
        mode |= Together_Mode3;
        break;
      case 4:
        mode |= Together_Mode4;
        break;
      case 5:
        mode |= Together_Mode5;
        break;
      case 6:
        mode |= Together_Mode6;
        break;
    }
    style = Together_isstyle(style);
  }

  if (mode != 0)
  {
    // The style is active.

    bm = bmcreate("Together_current");
    bm1 = bmcreate("Together_test");

    // Is the paragraph always to be at the top of a new page?

    if (mode & 32)
    {
      // Yes, this one is easy.

      setbmtocaret(bm);
      bmmove(bm, 0, 3);
      Together_insert_page(bm);
      bmmove(bm, 1, 3);
      setcarettobm(bm);
      type("\r");
    }
    else
    {
      // Are we to keep the paragraph together with the previous one?

      if (mode & 2)
      {
        // Yes, remember where we are.

        setbmtocaret(bm);
        bmmove(bm, 0, 3);

        // Move to the previous paragraph.

        setbmtobm(bm1, bm);
        bmmove(bm1, 0, 7);

        page = bmpage(bm1);
        if (bmpage(bm) - page == 1)
        {
          // It is on the previous page, move it onto the current one.

          Together_insert_page(bm1);
        }

        bmmove(bm, 1, 3);
        setcarettobm(bm);
      }

      // Are we to keep the paragraph together?

      if (mode & 4)
      {
        // Yes, remember where we are.

        setbmtocaret(bm);
        page = bmpage(bm);

        // Obtain the start of the paragraph.

        bmmove(bm, 0, 3);       // Start of current paragraph

        if (page - bmpage(bm) == 1)
        {
          // The start of the paragraph is on the previous page to the end.
          // Try to get then onto the same page.

          Together_insert_page(bm);
          bmmove(bm, 1, 3);
          setcarettobm(bm);
        }
      }

      if (mode & 16)
      {
        // Check for orphans.

        setbmtocaret(bm);
        bmmove(bm, 0, 3);       // Start of paragraph
        setbmtobm(bm1, bm);
        page = bmpage(bm);
        bmmove(bm, 1, 6);       // Start of next line
        bmmove(bm1, 1, 7);      // Start of next paragraph
        if (bmcmp(bm, bm1) < 0 && (bmpage(bm) - page == 1))
        {
          // The first line of the current paragraph is on a different page to the second.
          // Move the paragraph onto the next page.

          bmmove(bm, 0, 3);
          Together_insert_page(bm);
          bmmove(bm, 1, 3);
          setcarettobm(bm);
        }
      }

      if (mode & 8)
      {
        int bm2;

        // Check for widows.

        setbmtocaret(bm);
        page = bmpage(bm);      // End of current paragraph
        setbmtobm(bm1, bm);     // Remember where we are
        bmmove(bm, 0, 6);       // Start of previous line

        bmmove(bm1, 0, 3);      // Start of current paragraph
        bmmove(bm1, 0, 7);      // Start of previous paragraph
        bmmove(bm1, 1, 3);      // End of previous paragraph
        bmmove(bm1, 0, 2);      // Start of line at end of previous paragraph

        if (bmcmp(bm, bm1) > 0 && (page - bmpage(bm) == 1))
        {
          // The last line of the paragraph is on a different page to the next-to-last
          // line. Move the paragraph onto the next page.

          bmmove(bm, 0, 3);
          Together_insert_page(bm);
          bmmove(bm, 1, 3);
          setcarettobm(bm);
        }
      }

      // Are we to keep the paragraph together with the next one?

      if (mode & 1)
      {
        // Yes, remember where we are.

        setbmtocaret(bm);
        bmmove(bm, 0, 3);      // Start of paragraph
        page = bmpage(bm);

        // Do the new paragraph.

        type("\r");
        setbmtocaret(bm1);

        // Where are we now?

        if (bmpage(bm1) - page == 1)
        {
          // Different pages, try to get them on the same page.

          Together_insert_page(bm);
          bmmove(bm, 1, 7);
          setcarettobm(bm);
        }
      }
      else
        type("\r");
    }

    bmdelete(bm);
    bmdelete(bm1);

    return -1;
  }

  return key;
}

//---------------------------------------------------------------------------------------------

// Strips Together-inserted new pages for the specified chapter.
// If chapter is -1 then they are stripped for all chapters.

void Together_strip_pages(int chapter)
{
  int bm;
  int current;
  int low = Together_high_water;
  int high = 0;

  // Go through all possible together bookmarks.

  for (current = Together_low_water; current < Together_high_water; current++)
  {
    bm = bmfind("TogE" + itos(current));
    if (bm)
    {
      // Found one, but is it in the requested chapter?

      if (chapter <= -1 || bmchapter(bm) == chapter)
      {
        // It is, remove it and the following new page.

        setcarettobm(bm);
        if (caretchar() == 3) // If it is a new page ...
          type("{DeleteF}");
        bmdelete(bm);
      }
      else
      {
        if (current < low) low = current;
        if (current > high) high = current;
      }
    }
  }

  // Adjust the high and low water marks to what remains.

  if (low <= high)
  {
    Together_low_water = low;
    Together_high_water = high + 1;
  }
  else
  {
    Together_low_water = 1;
    Together_high_water = 1;
  }
}

//---------------------------------------------------------------------------------------------

// Locates the next new page that we have inserted and moves the caret to it.

void Together_next_location()
{
  int bm = 0;

  if (Together_current_bm < Together_low_water)
    Together_current_bm = Together_low_water;

  // Locate the next Together bookmark.

  while (!bm && Together_current_bm < Together_high_water)
  {
    bm = bmfind("TogE" + itos(Together_current_bm));
    Together_current_bm++;
    Together_current_bm_index = bm;
  }

  if (bm)
  {
    // Found one, move to it.

    setcarettobm(bm);
  }
  else
  {
    // Did not find one, bell and reset the current location.

    bbc_vdu(7);
    Together_current_bm = 0;
  }
}

//---------------------------------------------------------------------------------------------

// Scans through a chapter fixing up the keep with next/previous requests.
// tempbm should be located at the start of the chapter. Returns 0 if there
// are more chapters to do, and tempbm is left positioned at the start of the
// next chapter.

int Together_styletest(int tempbm)
{
  int style;
  int page;
  int i = 1;
  int bm;
  int bm1;
  int mode;

  bm = bmcreate ("Together_test");       // Bookmark for testing the current paragraph
  bm1 = bmcreate("Together_paracheck");  // Bookmark for testing paragraph starts/ends

  while (i != 0)                         // While not the end of text
  {
    setcarettobm(tempbm);                // Move the caret to the bookmark

    mode = 0;
    style = 0;

    style = Together_isstyle(style);   // Find out what style it is, and
                                       // do I need it
    while (style != 0)
    {
      switch (style)
      {
        case 1:
          mode |= Together_Mode1;
          break;
        case 2:
          mode |= Together_Mode2;
          break;
        case 3:
          mode |= Together_Mode3;
          break;
        case 4:
          mode |= Together_Mode4;
          break;
        case 5:
          mode |= Together_Mode5;
          break;
        case 6:
          mode |= Together_Mode6;
          break;
      }
      style = Together_isstyle(style);   // Find out what style it is, and
    }

    if (mode != 0)
    {
      if (mode & 32)
      {
        // The paragraph must always be at the top of a page.

        Together_insert_page(tempbm);
      }
      else
      {
        if (mode & 2)
        {
          // Keep together with the previous paragraph.

          setbmtobm(bm, tempbm);
          bmmove(bm, 0, 7);
          page = bmpage(tempbm);
          if (page - bmpage(bm) == 1)
          {
            // The previous paragraph is on the previous page, move it onto the current one.

            Together_insert_page(bm);
          }
        }

        if (mode & 4)
        {
          // Keep the current paragraph together.

          page = bmpage(tempbm);
          setbmtobm(bm, tempbm);
          bmmove(bm, 1, 3);             // End of current paragraph
          if (bmpage(bm) - page == 1)
          {
            // The end is on the next page, move the current paragraph onto the next
            // page.

            Together_insert_page(tempbm);
          }
        }

        if (mode & 16)
        {
          // Check for orphans.

          page = bmpage(tempbm);
          setbmtobm(bm, tempbm);
          bmmove(bm, 1, 6);              // Start of next line
          setbmtobm(bm1, tempbm);
          bmmove(bm1, 1, 7);             // Start of next paragraph
          if (bmcmp(bm, bm1) < 0 && bmpage(bm) - page == 1)
          {
            // The first line of the paragraph is on a different page to the second.
            // Move the paragraph onto the next page.

            Together_insert_page(tempbm);
          }
        }

        if (mode & 8)
        {
          // Check for widows.

          setbmtobm(bm, tempbm);
          setbmtobm(bm1, bm);   // Remember where we are
          bmmove(bm, 1, 3);     // End of current paragraph
          page = bmpage(bm);
          bmmove(bm, 0, 6);     // Start of previous line

          bmmove(bm1, 0, 7);    // Start of previous paragraph
          bmmove(bm1, 1, 3);    // End of previous paragraph
          bmmove(bm1, 0, 2);    // Start of line at end of previous paragraph

          if (bmcmp(bm, bm1) > 0 && page - bmpage(bm) == 1)
          {
            // The last line of the paragraph is on a different page to the next-to-last
            // line. Move the paragraph onto the next page.

            Together_insert_page(tempbm);
          }
        }

        if (mode & 1)
        {
          // Keep together with the next paragraph.

          page = bmpage(tempbm);
          setbmtobm(bm, tempbm);
          bmmove(bm, 1, 7);              // Start of next paragraph
          if (bmpage(bm) - page == 1)
          {
            // It is on the next page, move the current paragraph onto the next
            // page.

            Together_insert_page(tempbm);
          }
        }
      }
    }

    // Next paragraph.

    setbmtobm (bm, tempbm);			// Remember where we are
    i = bmmove(tempbm, 1, 7);       		// Move down a paragraph
    if (i != 0 && bmcmp(tempbm,bm) <= 0) i = 0;	// If we did not move, finish story
  }

  bmdelete(bm);
  bmdelete(bm1);

  i = bmnextstory(tempbm);		 // Move to next story

  return i;
}

//---------------------------------------------------------------------------------------------

// Do the scan (all chapters if requested).

void Together_do_together()
{
  int bm;

//  swi13(0x406c0, 0, 0, 0);

  bm = bmcreate("Together_SFind");       // Used when searching through the text
  bmstartscan();
  Together_set_styles();

  if (Together_all_chapters)
  {
    bmnextstory(bm);
    bmmove(bm, 0, 4);			 // Make sure we are at the start of the text
    setcarettobm(bm);			 // Start at beginning of document
    Together_strip_pages(-1);            // Remove any existing together new pages
    while (!Together_styletest(bm));     // Do all chapters
  }
  else
  {
    setbmtocaret(bm);
    bmmove(bm, 0, 4);                    // Move to start of current chapter
    Together_strip_pages(bmchapter(bm)); // Remove any existing together new pages
    Together_styletest(bm);              // Do the current chapter only
  }

  bmdelete (bm);

//  swi13(0x406c1, 0, 0, 0);
}

//---------------------------------------------------------------------------------------------

// Checks if the region to be deleted specified by bm1 to bm2 contains any of our
// bookmarks, and if so tidies things up.

void Together_check_delete(int bm1, int bm2)
{
  int current;
  int low = Together_high_water;
  int high = 0;
  int bm;
  int changed;

  // Go through all of our bookmarks, and get rid of any in the region.

  for (current = Together_low_water; current < Together_high_water; current++)
  {
    bm = bmfind("TogE" + itos(current));
    if (bm)
    {
      // Found one, but is it in the region?

      if (bmcmp(bm, bm1) >= 0 && bmcmp(bm, bm2) <= 0)
      {
        // It is, remove it, but leave any following new page so as not
        // to confuse the user.

        bmdelete(bm);
      }
      else
      {
        if (current < low) low = current;
        if (current > high) high = current;
      }
    }
  }

  // Adjust the high and low water marks to what remains.

  if (low <= high)
  {
    changed = (Together_low_water != low) | (Together_high_water != high + 1);
    Together_low_water = low;
    Together_high_water = high + 1;
  }
  else
  {
    changed = (Together_low_water != 1) | (Together_high_water != 1);
    Together_low_water = 1;
    Together_high_water = 1;
  }

  if (changed)
  {
    // The low or high water marks have changed, save them.

    Together_save_options();
  }
}

//---------------------------------------------------------------------------------------------

// Called by 7!bm when a region is cut or deleted. The current zone
// must be set to the region.

int Together_bmdeleter(int cut)
{
  int bm1;
  int bm2;

  // We need to check if any of our bookmarks are in the region.

  if (Together_low_water < Together_high_water && zoneorder() != 0)
  {
    bm1 = bmcreate("TogE_zonestart");
    bm2 = bmcreate("TogE_zoneend");
    setbmtozone(bm1, bm2);
    Together_check_delete(bm1, bm2);
    bmdelete(bm1);
    bmdelete(bm2);
  }

  return 0;       // Never claim the event so that other applets can have a go
}

//---------------------------------------------------------------------------------------------

// Called by 7!bm when a delete key is pressed.

int Together_delkeypress(int key)
{
  int bm1;
  int bm2;

  if (Together_low_water < Together_high_water)
  {
    switch (key)
    {
      case 127:                    // Delete <
      case 6664:
        bm1 = bmcreate("TogE_keystart");
        bm2 = bmcreate("TogE_keyend");
        setbmtocaret(bm1);
        setbmtobm(bm2, bm1);
        bmmove(bm1, 0, 0);         // Previous character
        Together_check_delete(bm1, bm2);
        bmdelete(bm1);
        bmdelete(bm2);
        break;

      case 395:                    // Delete >
        bm1 = bmcreate("TogE_keystart");
        bm2 = bmcreate("TogE_keyend");
        setbmtocaret(bm1);
        setbmtobm(bm2, bm1);
        bmmove(bm2, 1, 0);         // Next character
        Together_check_delete(bm1, bm2);
        bmdelete(bm1);
        bmdelete(bm2);
        break;

      // Other cases are managed by 7!bm, which selects the approriate zone
      // and calls Together_bmdeleter.

/*
      case 2175:                   // Delete word before cursor
      case 2568:
        setbmtocaret(bm1);
        bmmove(bm1, 0, 5);         // Start of previous word
        setbmtobm(bm2, bm1);
        bmmove(bm2, 1, 1);         // End of that word
        Together_check_delete(bm1, bm2);
        break;

      case 411:                    // Delete word at cursor
        setbmtocaret(bm1);
        setbmtobm(bm2, bm1);
        bmmove(bm1, 0, 1);         // Start of word
        bmmove(bm2, 1, 1);         // End of word
        Together_check_delete(bm1, bm2);
        break;
*/

      default:
        break;
    }
  }

  return key;
}

//=============================================================================================

////////////////////////////////////////////////
//  Together Style Menu Functions             //
////////////////////////////////////////////////


//---------------------------------------------------------------------------------------------

int Together_choosestyle()
{
  int i = 0;
  string tempstring = "";

  if(Together_stylemenu)
  {
    delete_menu(Together_stylemenu);
    Together_stylemenu = create_menu("{TOGETHER_05}");
  }

  // Add all the menu entries.

  addentry_menu (Together_stylemenu, "Together_styleentry", "Together_styleflags", "", "", "Clear Style");

  for(i = 0; getuserstyle(i, tempstring, 0) != 0; i++)
  {
    addentry_menu(Together_stylemenu, "Together_styleentry", "Together_styleflags", "", "", tempstring);
  }
}

//---------------------------------------------------------------------------------------------

int Together_styleentry (int n, int subcode)
{
  string stylename;

  if (n > 0)
    getuserstyle (n-1, stylename, 0);
  else
    stylename = "";		// Clear style

  switch (Together_current_icon)
  {
    case 6:
      Together_temp1 = n-1;
      writeicon (Together_win, 60, stylename);
      break;

    case 7:
      Together_temp2 = n-1;
      writeicon (Together_win, 63, stylename);
      break;

    case 8:
      Together_temp3 = n-1;
      writeicon (Together_win, 4, stylename);
      break;

    case 9:
      Together_temp4 = n-1;
      writeicon (Together_win, 5, stylename);
      break;

    case 61:
      Together_temp5 = n-1;
      writeicon (Together_win, 55, stylename);
      break;

    case 54:
      Together_temp6 = n-1;
      writeicon (Together_win, 67, stylename);
      break;

    default:
      return (0);
      break;
  }

  Together_check_ready();	// Check if ready to do Together now

  return (1);
}

//---------------------------------------------------------------------------------------------

// Tick, shade etc the items appropriately.

int Together_styleflags (int n, string &s)
{
  int bits = 0;

  switch (Together_current_icon)   // Find out which icon we are editing
  {
    case 6:
      if(Together_temp1 == n-1)    // Is the current style = this menu item
        bits = TICKED;             // if so then tick it
      break;

    case 7:
      if(Together_temp2 == n-1)    // and so on...
        bits = TICKED;
      break;

    case 8:
      if(Together_temp3 == n-1)
        bits = TICKED;
      break;

    case 9:
      if(Together_temp4 == n-1)
        bits = TICKED;
      break;

    case 61:
      if(Together_temp5 == n-1)
        bits = TICKED;
      break;

    case 54:
      if(Together_temp6 == n-1)
        bits = TICKED;
      break;

    default:
      return (0);
      break;
  }

  return (bits);   // Return the flags code
}

//=============================================================================================

////////////////////////////////////////////////
// Style Editing Window Code                  //
////////////////////////////////////////////////

// Load the default options.

void Together_load_defaults()
{
  Together_Style1 = -1;
  Together_Style2 = -1;
  Together_Style3 = -1;
  Together_Style4 = -1;
  Together_Style5 = -1;
  Together_Style6 = -1;
  Together_temp1 = -1;
  Together_temp2 = -1;
  Together_temp3 = -1;
  Together_temp4 = -1;
  Together_temp5 = -1;
  Together_temp6 = -1;
  Together_Style1s = "";
  Together_Style2s = "";
  Together_Style3s = "";
  Together_Style4s = "";
  Together_Style5s = "";
  Together_Style6s = "";
  Together_Mode1_temp = 0;
  Together_Mode2_temp = 0;
  Together_Mode3_temp = 0;
  Together_Mode4_temp = 0;
  Together_Mode5_temp = 0;
  Together_Mode6_temp = 0;
  Together_as_you_type = 1;
  Together_all_chapters = 0;
  Together_high_water = 1;
  Together_low_water = 1;
}

//---------------------------------------------------------------------------------------------

// Loads the saved options from the document. Returns true always.

int Together_load_document()
{
  Together_load_defaults();			// Set default values initially

  if (tagexists(Together_current_file, Together_tag+0, 0))
    Together_as_you_type = tagread (Together_current_file, Together_tag+0, 0);
  if (tagexists(Together_current_file, Together_tag+1, 0))
    Together_all_chapters = tagread (Together_current_file, Together_tag+1, 0);
  if (tagexists(Together_current_file, Together_tag+2, 0))
    Together_high_water = tagread (Together_current_file, Together_tag+2, 0);
  if (tagexists(Together_current_file, Together_tag+3, 0))
    Together_low_water = tagread (Together_current_file, Together_tag+3, 0);
  if (tagexists(Together_current_file, Together_tag+4, 0))
    tagreads (Together_current_file, Together_tag+4, Together_Style1s);
  if (tagexists(Together_current_file, Together_tag+5, 0))
    Together_Mode1 = tagread (Together_current_file, Together_tag+5, 0);
  if (tagexists(Together_current_file, Together_tag+6, 0))
    tagreads (Together_current_file, Together_tag+6, Together_Style2s);
  if (tagexists(Together_current_file, Together_tag+7, 0))
    Together_Mode2 = tagread (Together_current_file, Together_tag+7, 0);
  if (tagexists(Together_current_file, Together_tag+8, 0))
    tagreads (Together_current_file, Together_tag+8, Together_Style3s);
  if (tagexists(Together_current_file, Together_tag+9, 0))
    Together_Mode3 = tagread (Together_current_file, Together_tag+9, 0);
  if (tagexists(Together_current_file, Together_tag+10, 0))
    tagreads (Together_current_file, Together_tag+10, Together_Style4s);
  if (tagexists(Together_current_file, Together_tag+11, 0))
    Together_Mode4 = tagread (Together_current_file, Together_tag+11, 0);
  if (tagexists(Together_current_file, Together_tag+12, 0))
    tagreads (Together_current_file, Together_tag+12, Together_Style5s);
  if (tagexists(Together_current_file, Together_tag+13, 0))
    Together_Mode5 = tagread (Together_current_file, Together_tag+13, 0);
  if (tagexists(Together_current_file, Together_tag+14, 0))
    tagreads (Together_current_file, Together_tag+14, Together_Style6s);
  if (tagexists(Together_current_file, Together_tag+15, 0))
    Together_Mode6 = tagread (Together_current_file, Together_tag+15, 0);

  // Enable or remove the keypress event handler if we need to.

  if (Together_as_you_type && ((Together_Style1s != "" && Together_Mode1 != 0) || (Together_Style2s != "" && Together_Mode2 != 0)
                            || (Together_Style3s != "" && Together_Mode3 != 0) || (Together_Style4s != "" && Together_Mode4 != 0)
                            || (Together_Style5s != "" && Together_Mode5 != 0) || (Together_Style6s != "" && Together_Mode6 != 0)))
  {
    if (!Together_eh_installed)
    {
      addeventhandler(0x300, 0, "Together_keypress");
      Together_eh_installed = 1;
    }
  }
  else if (Together_eh_installed)
  {
    remeventhandler(0x300, 0, "Together_keypress");
    Together_eh_installed = 0;
  }

  Together_styles_set = 0;

  return 1;
}

//---------------------------------------------------------------------------------------------

// Sets the current options from the dialogue box.

void Together_set_options (int winhandle)
{
  Together_Style1 = Together_temp1;          // Permanently set the values for the styles
  Together_Style2 = Together_temp2;
  Together_Style3 = Together_temp3;
  Together_Style4 = Together_temp4;
  Together_Style5 = Together_temp5;
  Together_Style6 = Together_temp6;
  if (Together_Style1 >= 0)
    getuserstyle (Together_Style1, Together_Style1s, 0);
  else
    Together_Style1s = "";
  if (Together_Style2 >= 0)
    getuserstyle (Together_Style2, Together_Style2s, 0);
  else
    Together_Style2s = "";
  if (Together_Style3 >= 0)
    getuserstyle (Together_Style3, Together_Style3s, 0);
  else
    Together_Style3s = "";
  if (Together_Style4 >= 0)
    getuserstyle (Together_Style4, Together_Style4s, 0);
  else
    Together_Style4s = "";
  if (Together_Style5 >= 0)
    getuserstyle (Together_Style5, Together_Style5s, 0);
  else
    Together_Style5s = "";
  if (Together_Style6 >= 0)
    getuserstyle (Together_Style6, Together_Style6s, 0);
  else
    Together_Style6s = "";
  Together_Mode1 = Together_Mode1_temp;
  Together_Mode2 = Together_Mode2_temp;
  Together_Mode3 = Together_Mode3_temp;
  Together_Mode4 = Together_Mode4_temp;
  Together_Mode5 = Together_Mode5_temp;
  Together_Mode6 = Together_Mode6_temp;

  Together_as_you_type = Together_as_you_type_temp;
  Together_all_chapters = Together_all_chapters_temp;

  // Add or remove the keypress event handler depending on the state
  // of the as-you-type option.

  if (Together_as_you_type)
  {
    if (!Together_eh_installed)
    {
      addeventhandler(0x300, 0, "Together_keypress");
      Together_eh_installed = 1;
    }
  }
  else if (Together_eh_installed)
  {
    remeventhandler(0x300, 0, "Together_keypress");
    Together_eh_installed = 1;
  }
}

//---------------------------------------------------------------------------------------------

// Display the Together window.

int Together_StyleWin()
{
  int i;

  writeicon (Together_win, 60, Together_Style1s);
  writeicon (Together_win, 63, Together_Style2s);
  writeicon (Together_win, 4, Together_Style3s);
  writeicon (Together_win, 5, Together_Style4s);
  writeicon (Together_win, 55, Together_Style5s);
  writeicon (Together_win, 67, Together_Style6s);
  Together_Style1 = Together_findstyle(Together_Style1s);
  Together_Style2 = Together_findstyle(Together_Style2s);
  Together_Style3 = Together_findstyle(Together_Style3s);
  Together_Style4 = Together_findstyle(Together_Style4s);
  Together_Style5 = Together_findstyle(Together_Style5s);
  Together_Style6 = Together_findstyle(Together_Style6s);
  Together_temp1 = Together_Style1;
  Together_temp2 = Together_Style2;
  Together_temp3 = Together_Style3;
  Together_temp4 = Together_Style4;
  Together_temp5 = Together_Style5;
  Together_temp6 = Together_Style6;
  Together_Mode1_temp = Together_Mode1;
  Together_Mode2_temp = Together_Mode2;
  Together_Mode3_temp = Together_Mode3;
  Together_Mode4_temp = Together_Mode4;
  Together_Mode5_temp = Together_Mode5;
  Together_Mode6_temp = Together_Mode6;
  for (i = 0; i < 6; ++i)
    select_icon(Together_win, i + 17, Together_Mode1 & (1 << i));
  for (i = 0; i < 6; ++i)
    select_icon(Together_win, i + 23, Together_Mode2 & (1 << i));
  for (i = 0; i < 6; ++i)
    select_icon(Together_win, i + 29, Together_Mode3 & (1 << i));
  for (i = 0; i < 6; ++i)
    select_icon(Together_win, i + 35, Together_Mode4 & (1 << i));
  for (i = 0; i < 6; ++i)
    select_icon(Together_win, i + 41, Together_Mode5 & (1 << i));
  for (i = 0; i < 6; ++i)
    select_icon(Together_win, i + 47, Together_Mode6 & (1 << i));

  Together_as_you_type_temp = Together_as_you_type;
  select_icon(Together_win, 69, Together_as_you_type);

  Together_all_chapters_temp = Together_all_chapters;
  select_icon(Together_win, 57, Together_all_chapters);

  Together_check_ready();

  display_window(Together_win, 20, 1);

  return(1);
}

//---------------------------------------------------------------------------------------------

// Opens the menus off the buttons on the Together style window.

void Together_openwinmenu (int winhandle, int icon)
{
  Together_current_icon = icon;  // Set current icon
  Together_choosestyle();        // Make the menu
  open_menu(Together_stylemenu); // Show the menu
}

//---------------------------------------------------------------------------------------------



void Together_close(int handle)
{
 if(Together_win)
 {
  close_window(Together_win);
  delete_window(Together_win);
  Together_win=0;
  Together_current_file=0;
  Together_current_view=0;
 }
}




//  Mouse click event handler for the Together style window

void Together_click (int winhandle, int icon, int bbits, int mx, int my)
{
  int tempbm;
  int state;

  if (bbits != 4 && bbits != 1) return;	// Return if not left or right mouse button

  switch(icon)
  {
    case 1:                    // Default action window, do a scan

      if (Together_check_ready())
      {
        Together_set_options(winhandle);
        Together_save_options();
        Together_do_together();
        Together_save_options();  // In case high or low water marks changed
        Together_current_bm = 0;
        Together_check_any_pages();

        if (bbits == 4) Together_close(winhandle);
      }
      break;

    case 2:                   // Cancel button  - closes window
      Together_close(winhandle);
      Together_current_bm = 0;
      break;

    case 16:                 // Set values
      Together_set_options(winhandle);
      Together_save_options();
      Together_current_bm = 0;
      if (bbits == 4) Together_close(winhandle);
      break;

    case 6:                    // All the menu buttons
    case 7:
    case 8:
    case 9:
    case 61:
    case 54:
      // Open the menu off the menu buttons.
      Together_openwinmenu (winhandle, icon);
      break;

    case 17:                   // Style 1 options
    case 18:
    case 19:
    case 20:
    case 21:
    case 22:
      state = ~Together_Mode1_temp & (1 << (icon - 17));
      Together_Mode1_temp = (Together_Mode1_temp & ~(1 << (icon - 17))) | state;
      select_icon(Together_win, icon, state);
      Together_check_ready();
      break;

    case 23:                   // Style 2 options
    case 24:
    case 25:
    case 26:
    case 27:
    case 28:
      state = ~Together_Mode2_temp & (1 << (icon - 23));
      Together_Mode2_temp = (Together_Mode2_temp & ~(1 << (icon - 23))) | state;
      select_icon(Together_win, icon, state);
      Together_check_ready();
      break;

    case 29:                   // Style 3 options
    case 30:
    case 31:
    case 32:
    case 33:
    case 34:
      state = ~Together_Mode3_temp & (1 << (icon - 29));
      Together_Mode3_temp = (Together_Mode3_temp & ~(1 << (icon - 29))) | state;
      select_icon(Together_win, icon, state);
      Together_check_ready();
      break;

   case 35:                   // Style 4 options
   case 36:
   case 37:
   case 38:
   case 39:
   case 40:
      state = ~Together_Mode4_temp & (1 << (icon - 35));
      Together_Mode4_temp = (Together_Mode4_temp & ~(1 << (icon - 35))) | state;
      select_icon(Together_win, icon, state);
      Together_check_ready();
      break;

   case 41:                   // Style 5 options
   case 42:
   case 43:
   case 44:
   case 45:
   case 46:
      state = ~Together_Mode5_temp & (1 << (icon - 41));
      Together_Mode5_temp = (Together_Mode5_temp & ~(1 << (icon - 41))) | state;
      select_icon(Together_win, icon, state);
      Together_check_ready();
      break;

   case 47:                   // Style 6 options
   case 48:
   case 49:
   case 50:
   case 51:
   case 52:
      state = ~Together_Mode6_temp & (1 << (icon - 47));
      Together_Mode6_temp = (Together_Mode6_temp & ~(1 << (icon - 47))) | state;
      select_icon(Together_win, icon, state);
      Together_check_ready();
      break;

    case 69:                   // As-you-type option
      Together_as_you_type_temp = !Together_as_you_type_temp;
      select_icon(Together_win, 69, Together_as_you_type_temp);
      break;

    case 57:                   // All chapters option
      Together_all_chapters_temp = !Together_all_chapters_temp;
      select_icon(Together_win, 57, Together_all_chapters_temp);
      break;

    case 62:                   // Find the next new page inserted by us
      Together_next_location();
      Together_check_any_pages();
      break;

    case 71:                   // Remove the current new page as found by Find
      if (Together_current_bm > 0)
      {
        setcarettobm(Together_current_bm_index);
        bmdelete(Together_current_bm_index);
        if (caretchar() == 3) // If it is a new page ...
          type("{DeleteF}");
        else
          bbc_vdu(7);
      }
      else
        bbc_vdu(7);
      set_icon_state(Together_win, 51, 0x00400000, 0x00400000);
      break;

    case 58:                   // Remove all new pages inserted by us
      if (Together_all_chapters_temp)
        Together_strip_pages(-1);
      else
      {
        if (caretchar() >= 0)
          Together_strip_pages(currentchapter(Together_current_view));
        else
          bbc_vdu(7);
      }
      Together_current_bm = 0;
      Together_check_any_pages();
      break;

    default:                   // Click anywhere else - don't do anything
      break;

  }
}

//---------------------------------------------------------------------------------------------

// Key press event handler for Together style window.

int Together_key_press (int winhandle, int icon, int key)
{
  switch (key)
  {
    case 13:			// Return, simulate mouse click over Scan
      Together_click (winhandle, 1, 4, 0, 0);
      break;

    case 27:			// Esc, simulate mouse click over Cancel
      Together_click (winhandle, 2, 4, 0, 0);
      break;

    default:
      break;
   }

   return key;
}

//=============================================================================================

////////////////////////////////////////////////
// Together Menu Functions                    //
////////////////////////////////////////////////

// Called when the Together menu entry is clicked.

int Together_command (int n, int sub)
{

 Together_close(Together_win);

  // Check if the file has changed.

  if (Together_current_file != currentfile())
  {
    Together_current_file = currentfile();	// Get file handle
    Together_load_document ();			// Load options saved with document
  }

  Together_current_view=currentview();

  // Add handlers for when the user clicks on the Together styles window or when
  // s(he) hits a key with it displayed.


  Together_win = create_windowparent("Together",viewhandle(currentview()));

  addwindowhandler(0, Together_win, "Together_click");
  addwindowhandler(1, Together_win, "Together_key_press");
  addwindowhandler(7, Together_win, "Together_close");
  addwindowhandler(2, Together_win, "Together_close");

  Together_StyleWin();				// Display the Together window

  setactive(Together_win);
}

//=============================================================================================

// Called when a file is closed.

void Together_file_closing (int user, int file)
{
  if (file == Together_current_file)
  {
    Together_close(Together_win);
  }
}

//---------------------------------------------------------------------------------------------

// Called when a view is made active.

void Together_view_active (int user, int view, int gain)
{
  if (view == Together_current_view)
  {
    if (!gain)
    {
      Together_close(Together_win);
    }
  }
}

//---------------------------------------------------------------------------------------------

// Called when a view is closed.

void Together_view_closed (int user, int view)
{
  if (view == Together_current_view)
  {
    Together_close(Together_win);
  }
}

//=============================================================================================

////////////////////////////////////////////////
// Main                                       //
////////////////////////////////////////////////

void main(void)
{
  string help = "{TOGETHER_10}";

  Together_load_defaults ();	// Set the default options

  // Create the menus and windows, assign to handles.

  Together_stylemenu = create_menu("{TOGETHER_05}");


  // Add Together menu to applets menu.

  script_menu_initialise();             // Initialise the menu
  script_menu_insert("Together_command", "", "", "", "{TOGETHER_00}");


  // Add handlers for files and views being closed and views made active.

  addeventhandler (0x02, 0, "Together_file_closing");
  addeventhandler (0x06, 0, "Together_view_closed");
  addeventhandler (0x215,0, "Together_view_active");

  // Define a macro to launch us off.

  if (isdefmacro("Together") == -1)
  {
    translate(help);
    defmacro3(0, "Together", "{Together_command(0,0)}", "Together", help, "Together");
  }

  // Declare deletion handlers so that we can tidy up our bookmarks in deleted text.
  // We can only do this if !!!!!!!bm is being used. In this case we also take advantage
  // of that applet to hide our bookmarks. We do this as a macro so that it does not
  // cause interpreter errors if the bm_ functions do not exist.

//  if (objectexists("<OvationPro$Applets>.!!!!!!!bm") == 2)
//  {
//     macro("{bm_hide(\"TogE\");bm_registerdelfn(\"Together_bmdeleter\");bm_registerdelkeyfn(\"Together_delkeypress\");}");
//  }
}
