/*->c.ostyle     */

/* Ovation!   (c) D. J. Pilling,  December 1988                     */
/*                   Filing System  Interface                         */

#include "stdafx.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <ctype.h>
#include <time.h>


#include "os.h"
#include "wimp.h"
#include "sprite.h"
#include "bbc.h"
#include "akbd.h"
#include "flex.h"
#include "werr.h"
#include "font.h"


#include "transform.h"
#include "fsx.h"
#include "bf.h"
#include "wos.h"
#include "etc.h"
#include "bits.h"
#include "xmath.h"
#include "conf.h"
#include "trans.h"

#include "str.h"
#include "constants.h"

#include "file.h"

#include "xcolour.h"


#include "txstr.h"
#include "fox.h"
#include "style.h"
#include "ustyle.h"
#include "tabs.h"
#include "fmap.h"
#include "codes.h"
#include "tx.h"
#include "text.h"


#include "odef.h"
#include "ostyle.h"







typedef struct mapstr
{
 int old;
 int xnew;
} mapstr;



#define MAPCHUNK 64*sizeof(mapstr)

static mapstr * absmap;
static int      absentries;
static mapstr * usermap;
static int      userentries;
static mapstr * tabmap;
static int      tabentries;
static mapstr * xfontmap;
static int      fontentries;




/* look up old number in map, and return new number or 0 */

static int cmpmap(const void * p1,const void * p2)
{
 mapstr * q1;
 mapstr * q2;
 q1=(mapstr *)p1;
 q2=(mapstr *)p2;
 return(q1->old-q2->old);
}



static int mapfind(int old,mapstr * map,int max)
{
 mapstr * r;
 mapstr   maphdr;

 maphdr.old=old;

 r=(mapstr*)bsearch(&maphdr,map,max,sizeof(mapstr),cmpmap);
 if(r) return(r->xnew);
 else  return(0);
}



static os_error * mapadd(int old,int xnew,mapstr ** map,int * maxp)
{
 os_error * err;
 mapstr   * r;
 mapstr     maphdr;
 int        n;
 int        max;

 max=*maxp;

 err=flex_chunk((flex_ptr)map,sizeof(mapstr)*(max+1),MAPCHUNK);

 maphdr.old=old;

 r=(mapstr*)bnearest(&maphdr,*map,max,sizeof(mapstr),cmpmap);

 n=r-(*map);
 memmove((*map)+n+1,(*map)+n,sizeof(mapstr)*(max-n));
 (*map)[n].old=old;
 (*map)[n].xnew=xnew;

 (*maxp)++;

 return(err);
}




static void trashmaps(void)
{
 flex_free((flex_ptr)&absmap);                                                   flex_free((flex_ptr)&usermap);
 flex_free((flex_ptr)&tabmap);
 flex_free((flex_ptr)&xfontmap);
}


static os_error * initmaps(void)
{
 os_error * err;

 absentries=0;
 userentries=0;
 tabentries=0;
 fontentries=0;

          err=flex_alloc((flex_ptr)&absmap,MAPCHUNK);
 if(!err) err=flex_alloc((flex_ptr)&usermap,MAPCHUNK);
 if(!err) err=flex_alloc((flex_ptr)&tabmap,MAPCHUNK);
 if(!err) err=flex_alloc((flex_ptr)&xfontmap,MAPCHUNK);

 if(err) trashmaps();

 return(err);
}



static char * deffontnames[]=
{
 "Trinity.Medium",
 "SwissB.Medium",
 "Homerton.Medium",
 "Helvetica.Medium",
 "Helvetica"
};


static int odefltfont(void)
{
 int font;
 int i;

 for(i=0;i<(sizeof(deffontnames)/sizeof(char*));i++)
 {
  if((font=findxuserfont(deffontnames[i]))!=-1) break;
 }

 if(i==(sizeof(deffontnames)/sizeof(char*))) font=0;

 return(font);
}


#define MISSFONTSIZE 32

static char missingfontname[MISSFONTSIZE];
static int  missingfonts;

os_error * readfontmap(buffer * bf,filestr * file,int * fmcode)
{
 int  newfh;
 int  oldfh;
 int  usage;
 int  len;
 char string[128];
 int  deffh;

 missingfonts=0;
 initmaps();
 deffh=odefltfont();

 while(1)
 {
  bf_read(bf,&len,sizeof(int));

  if(!len) break;
  bf_read(bf,(char *)string,len+1);
  bf_read(bf,(char *)&usage,sizeof(int));
  bf_read(bf,(char *)&oldfh,sizeof(int));

  newfh=findxuserfont(string);
  if(newfh<0) 
  {
   newfh=deffh;
   if(!missingfonts) xstrncpy(missingfontname,string,MISSFONTSIZE);
   missingfonts++;
  }

  mapadd(oldfh,newfh,&xfontmap,&fontentries);
 }

 *fmcode=(missingfonts!=0);

 return(NULL);
 USE(file);
}


/* returns 0 if OK, or 1 if chuck away */

int domapfonts(filestr * file,int fmcode)
{
 int  code;
 char string[256];

 if(!fmcode) return(0);

 if(missingfonts==1) strcpy(string,"{FOX00}");
 else                strcpy(string,"{FOX01}");

 trans(string,sizeof(string));

 if(missingfonts==1) confirmparent(NULL,CONYN,&code,string,missingfontname);
 else                confirmparent(NULL,CONYN,&code,string,missingfonts,missingfontname);

 if(code==1)
 {
  reflowfile(file);
  return(0);
 }
 else
 {
  return(1);
 }
}






static ofileupstyles  fh_upstyle;      /* hold the user para styles   */
static ofileparastore fh_para;         /* hold para rulers            */
static ofilestyles    fh_style;        /* hold styles vectors         */



static colourvalue xtrans={OURTRANS};


static int seekpara(int pn,ofileparastore * fps)
{
 int         n=fps->noparas;
 int         lo=0;
 int         hi=(n-1)*sizeof(oparastore);
 int         i=0;
 int         loi=0;
 int         hii=n-1;
 int         ii=0;
 oparastore * ps=NULL;

 while(hi>=lo)
 {
  i=(lo+hi)>>1;
  ii=loi+hii;
  if((ii & 1) && i) i-=sizeof(oparastore)/2;
  ii=ii>>1;

  ps=(oparastore*)(((char*)fps->paras)+i);

  if(ps->para==pn) return(i);
  else
  if(ps->para<pn)  {lo=i+sizeof(oparastore);loi=ii+1;}
  else             {hi=i-sizeof(oparastore);hii=ii-1;}
 }

 if(!n) i=0;
 else
 if(ps->para<pn)    i+=sizeof(oparastore);

 return(-1-i);
}



static oparastore * ofindps(ofileparastore * fps,int pn)
{
 int offset=seekpara(pn,fps);
 if(offset<0) offset=0;
 return((oparastore *)(((char*)fps->paras)+offset));
}



#define DSTYLE     0x0FFF
#define DUSTYLE   0x10000
#define DFONT     0x20000
#define DXPTS     0x40000
#define DYPTS     0x80000
#define DTRACK   0x100000
#define DFCOL    0x200000

#define DTAB     0x400000
#define DJUST    0x800000
#define DLEAD   0x1000000
#define DAFTER  0x2000000
#define DBEFORE 0x4000000
#define DWWRAP  0x8000000
#define DHCAP  0x10000000
#define DHMIN  0x20000000
#define DHYPH  0x40000000
#define DAKERN 0x80000000

#define DLEFT      0x1000
#define DRIGHT     0x2000
#define DFIRST     0x4000

/* #define DPARA  0x7FC0F000  */
#define DPARA  0xFFC0F000




static os_error * convertstyle(xstylestr * s,oparastyle * ps,int bits,
                                               int * tabkey,filestr * file)
{
 int              style;
 int              flags;
 tabstr           tab[XMAXTABS];
 int              tkey;
 int              toff;
 int              xtoff;
 oparastore     * pss;
 tabstr         * xtabs;
 char           * otabs;
 int              j;
 colourvalue      cv;



 *tabkey=0;

 s->data.font=mapfind(ps->font,xfontmap,fontentries);

 if(bits & DFONT)
 {
  s->action.font=1;
 }

 if(bits & DXPTS)
 {
  s->data.xpts=ps->xpts;   /* xsize in kpts */
  s->action.hscale=1;
 }

 if(bits & DYPTS)
 {
  s->data.ypts=ps->ypts;   /* ysize in kpts */  
  s->action.size=1;
 }

 if(bits & DTRACK)
 {
  s->data.track=ps->track;  /* absolute tracking in kpts */
  s->action.track=1;
 }

 style=ps->style; /* style bits */

 if(bits & XBOLD)
 {
  s->data.bold=(style & XBOLD)!=0;
  s->action.bold=1;
 }

 if(bits & XITALICS)
 {
  s->data.italic=(style & XITALICS)!=0; 
  s->action.italic=1;
 }


 if(bits & XUNDER)
 {
  s->data.underline=(style & XUNDER)!=0;
  s->data.ulword=(style & XWUNDER)!=0;
  s->action.underline=1;
 }

 if(bits & XREVERSE)
 {
  s->data.reverse=(style & XREVERSE)!=0;
  s->action.reverse=1;
 }

 if(bits & (XSUBS|XSUPS))
 {
  if(style & XSUBS) s->data.supersub=2;
  else
  if(style & XSUPS) s->data.supersub=1;
  s->action.supersub=1;
 }

 if(bits & XWUNDER)
 {
  s->data.ulword=(style & XWUNDER)!=0;
  s->data.underline=s->data.ulword;
  s->action.underline=1;
 }

 if(bits & XSMLCAPS)
 {
  s->data.smlcaps=(style & XSMLCAPS)!=0;
  s->action.smlcaps=1;
 }

 if(bits & (XTITLE|XALLCAPS))
 {
  if(style & XTITLE)   s->data.xcase=TITLE;
  else
  if(style & XALLCAPS) s->data.xcase=ALLCAPS;
  s->action.xcase=1;
 }

 /* ups->ps.fcol; */  /* foreground colour                  */

 if(bits & DFCOL)
 {
  getcolour(ps->fcol,0,&cv,file);

  s->data.fcolour=cv;
  s->data.bcolour=xtrans;
  s->data.ulcolour=cv;
  s->data.stcolour=cv;
  s->data.vrulecolour=cv;
  s->data.hrulecolour=cv;

  s->action.fcolour=1;
  s->action.bcolour=1;
 }



 s->data.lead=ps->lead;   /* absolute leading for the font      */
 s->data.height=ps->height;

 pss=ofindps(&fh_para,ps->parano); /*our number for the paragraph style */
 if(pss)
 {
  flags=pss->flags;  /* flags  */

  if(bits & DLEAD)
  {
   if(flags & XLEADPC)
   {
    s->data.leadingtype=LEADPC;
    s->data.leading=scale(pss->leading,0x10000,100);
   }
   else
   {
    s->data.leading=pss->leading; /* leading in appropriate units */

    if(flags & XLEADDL)  s->data.leadingtype=LEADDL;
    else
    if(flags & XLEADABS) s->data.leadingtype=LEADABS;
   }

   s->action.leading=1;
  }

  if(bits & DJUST)
  {
   if(flags & XRIGHT)   s->data.align=RIGHT;
   else
   if(flags & XLEFT)    s->data.align=LEFT;
   else
   if(flags & XCENTRE)  s->data.align=CENTRE;
   else
   if(flags & XJUSTIFY) s->data.align=JUSTIFY;

   s->action.align=1;
  }

  if(bits & DWWRAP)
  {
   s->data.wordwrap=(flags & XWORDWRAP)!=0;
   s->action.wordwrap=1;
  }

  if(bits & (DHYPH|DHCAP|DHMIN))
  {
   s->data.hyphenation=(flags & XHYPH)!=0;
   s->data.hpbreakcaps=(flags & XHYCAPS)!=0;
   s->data.hpmin=pss->minwlen;    /* min word len for hyphenation */
   s->action.hyphenation=1;
  }

  if(bits & DAKERN)
  {
   s->data.akern=(flags & XAUTOKERN)!=0;
   s->action.akern=1;
  }


  if(bits & DLEFT)
  {
   s->data.firstshift+=s->data.leftshift;

   if(pss->left>=0)
   {
    s->data.leftshift=pss->left;        /* left indent                  */
   }
   else
   {
    s->data.leftshift=0;
   }
   s->action.left=1;

   s->data.firstshift-=s->data.leftshift;
   s->action.first=1;
  }

  if(bits & DRIGHT)
  {
   if(pss->right>=0)
   {
    s->data.rightshift=pss->right;      /* right indent                 */
   }
   else
   {
    s->data.rightshift=0;
   }
   s->action.right=1;
  }

  if(bits & DFIRST)
  {
   if(pss->first>=0)
   {
    s->data.firstshift=pss->first-s->data.leftshift; /* first line indent */
   }
   else
   {
    s->data.firstshift=0;
   }
   s->action.first=1;
  }


  if(bits & DBEFORE)
  {
   s->data.before=pss->before;         /* space before                 */
   s->action.before=1;
  }

  if(bits & DAFTER)
  {
   s->data.after=pss->after;           /* space after                  */
   s->action.after=1;
  }


/*  dprintf(0,"bits=%x DTAB=%x and=%d",bits,DTAB,bits & DTAB);
  dprintf(1,"ntab=%d toff=%d",pss->ntab,pss->toff); */

  if(bits & DTAB) 
  {
   s->action.tab=1;

   toff=pss->toff;
   xtoff=pss->ntab?toff:(-1);

   tkey=mapfind(xtoff,tabmap,tabentries);
   if(!tkey)
   {
    otabs=(((char*)fh_para.tabs)+toff);
    for(j=0;j<pss->ntab;j++)
    {
     tab[j].type=*otabs++;
     tab[j].posn=(*otabs)+((*(otabs+1))<<8)+
                 ((*(otabs+2))<<16)+((*(otabs+3))<<24);
     otabs+=4;
     tab[j].selected=0;
    }

    xtabs=tab;
    addtabruler(file,&xtabs,0,pss->ntab,tabkey);
    tkey=*tabkey;

    mapadd(xtoff,tkey,&tabmap,&tabentries);
   }

   s->data.tab=tkey;
  }
 }

 return(NULL);
}



static void o1getdefstyle(userstylestr * s)
{
 getdefstyle(s);
 s->style.data.lefttype=INDENTDELTA;
 s->style.data.righttype=INDENTDELTA;
}



static os_error * createstylemaps(filestr * file)
{
 os_error       * err;
 userstylestr     s;
 int              i;
 ouserparastyle   ups;
 int              uskey;
 int              fmkey;
 int              tabkey;


 err=NULL;

 memset(&s,0,sizeof(userstylestr));

 for(i=0;i<fh_upstyle.nostyles;i++)
 {
  ups=*(((ouserparastyle*)fh_upstyle.anchor)+i);

  o1getdefstyle(&s);
  stylesetall(&s.style);

  strcpy(s.name,ups.name);

  s.key=ups.key & 0xFFFF;
  if(!s.key) s.key=-1;

  convertstyle(&s.style,&ups.ps,-1,&tabkey,file);

  fmapadd(file,s.style.data.font,1,&fmkey);
  s.style.data.mfont=fmkey;

  adduserstyle2(&s,file,&uskey,1,0);

  if(tabkey) remtabruler(file,tabkey);
  fmaprem(file,fmkey,1);

  mapadd(i,uskey,&usermap,&userentries);
 }

 return(err);
}



static int seekstyle(int fn,ofilestyles * fs)
{
 int         n=fs->no;
 int         lo=0;
 int         hi=(n-1)*sizeof(ostyle);
 int         i=0;
 int         loi=0;
 int         hii=n-1;
 int         ii=0;
 ostyle *    f=NULL;

 while(hi>=lo)
 {
  i=(lo+hi)>>1;
  ii=loi+hii;
  if((ii & 1) && i) i-=sizeof(ostyle)/2;
  ii=ii>>1;

  f=(ostyle *)(((char*)fs->anchor)+i);

  if(f->number==fn) return(i);
  else
  if(f->number<fn)  {lo=i+sizeof(ostyle);loi=ii+1;}
  else              {hi=i-sizeof(ostyle);hii=ii-1;}
 }

 if(!n) i=0;
 else
 if(f->number<fn)      i+=sizeof(ostyle);

 return(-1-i);
}



static ostyle * fsfind(ofilestyles * fs,int sn)
{
 int offset=seekstyle(sn,fs);
 if(offset<0) offset=0;
 return((ostyle *)(((char*)fs->anchor)+offset));
}




static os_error * mapfstyle(int fs,int * nfs,filestr * file)
{
 int            xnew;
 ostyle       * os;
 absstylestr    s;
 userstylestr   user;
 int            chainkey;
 int            tabkey;
 int            chain[2];


 xnew=mapfind(fs,absmap,absentries);
 if(!xnew)
 {
  memset(&s,0,sizeof(absstylestr));

  os=fsfind(&fh_style,fs);
  xnew=mapfind(os->upstyleno,usermap,userentries);

  if(xnew==BODYTEXT)
  {
   addstylechain(file,&xnew,1,&chainkey,1);
  }
  else
  {
   chain[0]=xnew;
   chain[1]=BODYTEXT;
   addstylechain(file,chain,2,&chainkey,1);
  }

  o1getdefstyle(&user);
  s.style=user.style;
  styleclrall(&s.style);

  s.chain=chainkey;
  fillinstyle(&s,file);

  convertstyle(&s.style,&os->pstyle,os->bits,&tabkey,file);

  addabsstyle(file,&s,1);

  remstylechain(file,chainkey,1);
  if(tabkey) remtabruler(file,tabkey);

  mapadd(fs,s.style.n,&absmap,&absentries);
  xnew=s.style.n;
 }
 else
 {
  dupabsstyle(file,xnew);
 }

 *nfs=xnew;

 return(NULL);
}



static os_error * fxloadbuffer(obuffer ** anchor,buffer * bf) 
{
 os_error * err;
 obuffer    temp;
 int        blen;

 err=bf_read(bf,&temp,sizeof(obuffer)-4);

 blen=temp.len-sizeof(obuffer)+4;

 if(!err) err=flex_alloc((flex_ptr)anchor,blen);
 if(!err) err=bf_read(bf,*anchor,blen);

 return(err);
}



os_error * oclearstyles(void)
{
 trashmaps();

 flex_free((flex_ptr)&fh_upstyle.anchor);
 flex_free((flex_ptr)&fh_style.anchor);
 flex_free((flex_ptr)&fh_para.paras);
 return(flex_free((flex_ptr)&fh_para.tabs));
}




/* load the styles for this file */

os_error * loadstyles(buffer * bf,filestr * file)
{
 os_error * err;

          err=bf_read(bf,&fh_upstyle.nostyles,sizeof(int));
 if(!err) err=fxloadbuffer(&fh_upstyle.anchor,bf);

 return(err);

 USE(file);
}




/* load the fstyles for this file */
/* called last                    */


os_error * loadfstyles(buffer * bf,filestr * file)
{
 os_error * err;

          err=bf_read(bf,&fh_style.no,sizeof(int));
 if(!err) err=fxloadbuffer(&fh_style.anchor,bf);

 if(!err) err=createstylemaps(file);

 return(err);
}



/* load the paras for this file */

os_error * loadparas(buffer * bf,filestr * file)
{
 os_error * err;

          err=bf_read(bf,&fh_para.noparas,sizeof(int));
 if(!err) err=fxloadbuffer(&fh_para.paras,bf);
 if(!err) err=bf_read(bf,&fh_para.notabs,sizeof(int));
 if(!err) err=fxloadbuffer(&fh_para.tabs,bf);

 return(err);

 USE(file);
}



static char justmap[4]={ISLEFT,ISRIGHT,ISCENT,ISJUST};

os_error * oremapvsource(int vsource,filestr * file)
{
 os_error   * err;
 vsourcestr * vsourcep;
 int          len;
 vlstr      * vl;
 int          x;
 int          y;
 int          w;
 int          h;
 int          mw;
 int          s;
 int          ln;
 int          font;
 int          hyph;
 int          akern;
 int          eol;
 int          just;
 int          ns;
 int          i;


 err=fob_lock((fobjptr)&vsourcep,vsource,file);
 if(!err)
 {
  len=vsourcep->len;
  vl=(vlstr*)vsourcep->data;
  i=0;

  while(len)
  {
   x=XLX(((ovline*)vl));
   y=XLY(((ovline*)vl));
   h=XLH(((ovline*)vl));
   w=XLW(((ovline*)vl));
   mw=XLMW(((ovline*)vl));
   s=XLFS(((ovline*)vl));
   ln=XLLEN(((ovline*)vl));
   eol=XEOLCHAR(((ovline*)vl));
   hyph=XLHYPH(((ovline*)vl));
   font=XLFONT(((ovline*)vl));
   just=XLJ(((ovline*)vl));
   akern=XLAUTOKERN(((ovline*)vl));

   memset(vl,0,sizeof(vlstr));

   vl->line.x=x;
   vl->line.yy=y;
   vl->line.w=w;
   vl->line.h=h;

   vl->line.just=justmap[(just>>24)];

   vl->line.strip=1;
   vl->line.font=font;
   vl->line.eol=(hyph && eol==XISTEXT)?ISHYP:(eol>>28);

/*   if(eol==ISNL || eol==ISBREAK) vl->just=ISLEFT; */


   mapfstyle(s,&ns,file);
   vl=((vlstr*)vsourcep->data)+i;


   vl->line.style=ns;

   vl->line.len=ln;
   SETMAXW(vl,mw);

   vl++;
   len-=sizeof(vlstr);
   i++;
  }

  fob_unlock((fobjptr)&vsourcep,file);
 }

 return(err);
}


os_error * ovlineshift(filestr * file,int vsource,int lines,int start,
                                                            int dx,int dy)
{
 os_error   * err;
 vsourcestr * vsourcep;
 vlstr      * vl;

 err=fob_lock((fobjptr)&vsourcep,vsource,file);
 if(!err)
 {
  vl=(vlstr*)(vsourcep->data+start);
  while(lines)
  {
   vl->line.x+=dx;
   vl->line.yy+=dy;
   vl++;
   lines-=sizeof(vlstr);
  }
  fob_unlock((fobjptr)&vsourcep,file);
 }

 return(err);
}


os_error * oremaptxsource(int source,filestr * file)
{
 os_error   * err;
 sourcestr  * sourcep;
 int          sn;
 int          nsn;
 int          c;
 char       * p;
 int          offset;
 int          slen;
 int          bits;
 int          srclen;


 err=fob_lock((fobjptr)&sourcep,source,file);
 if(!err)
 {
  offset=0;
  srclen=sourcep->len;

  if(srclen)
  {
   p=NULL; /* compiler */

   while(offset<srclen)
   {
    p=txptr(sourcep,offset);
    c=*p;
    bits=codefn(c);
    slen=bits & CODELEN;
 
    if(c==SNO)
    {
     sn=(*(p+1))+((*(p+2))<<8)+((*(p+3))<<16);
     mapfstyle(sn,&nsn,file);

     p=txptr(sourcep,offset);
     *(p+1)=nsn;
     *(p+2)=nsn>>8;
     *(p+3)=nsn>>16;
    }
    else
    if(c==EOT)
    {
     *p=' ';
    }

    offset+=slen;
   }

   offset-=slen;
   *p=EOT;
  }

  fob_unlock((fobjptr)&sourcep,file);
 }

 return(err);
}



